/** \file send.h sender-related defines and structures */
/** the sender subcommands */
enum {SENDER_ADD, SENDER_DELETE, SENDER_ALLOW, SENDER_DENY, SENDER_ON, SENDER_OFF};

/** the number of sender subcommands */
#define NUM_SENDER_CMDS (SENDER_OFF + 1)

/**
 * describes one supported sender of para_server
 *
 * \sa http_send.c ortp_send.c
 */
struct sender {
/** the name of the sender */
	const char *name;
/**
 * the init function of this sender
 *
 * It must fill in all function pointers of \a s as well as the \a client_cmds
 * array, see below. It should also do all neccessary preparations to init
 * this sending facility, for example it could open a tcp port.
 */
	void (*init)(struct sender *s);
/** \p SENDER_ON or \p SENDER_OFF */
	int status;
/**
 * return the help text of this sender
 *
 * The result must be dynamically allocated and is freed by the caller.
 */
	char* (*help)(void);
/**
 * return current status info about this sender
 *
 * The result must be dynamically allocated and is freed by the caller.
 */
	char* (*info)(void);
/**
 * the send-hook
 *
 * It gets called whenever para_server is playing and the current
 * audio format handler indicates that another chunk of data should
 * be sent now. The two parameters \a current_chunk and \a chunks_sent
 * only differ if the stream was repositioned by the \a ff or \a jmp
 * command. Of course, \a buf is a pointer to the chunk of data which
 * should be sent, and \a len is the length of this buffer.
*/
	void (*send)(struct audio_format *af, long unsigned current_chunk,
		long unsigned chunks_sent, const char *buf, size_t len);
/** add file descriptors to fd_sets
 *
 * The pre_select function of each supported sender is called just before
 * para_server enters its main select loop. Each sender may add its own
 * file descriptors to the \a rfds or the \a wfds set.
 *
 * If a file descriptor was added, \a max_fileno must be increased by
 * this function, if neccessary.
 *
 * \sa select(2)
*/
	void (*pre_select)(struct audio_format *af, int *max_fileno, fd_set *rfds,
		fd_set *wfds);
/**
 * handle the file descriptors which are ready for I/O
 *
 * If the pre_select hook added one ore more file descriptors to the read or write
 * set, this is the hook to check the result and do any I/O on those descriptors
 * which are ready for reading/writing.
 */
	void (*post_select)(struct audio_format *af, fd_set *rfds, fd_set *wfds);
/**
 * terminate all connected clients
 *
 * This is called e.g. if the stop command was executed. It should make the clients
 * aware of the end-of-file condition.
 */
	void (*shutdown_clients)(void);
/**
 * array of function pointers for the sender subcommands
 *
 * Each sender may implement any subset of the sender commands by filling in
 * the aprropriate function pointer in the array. A \p NULL pointer means this
 * command is not implemented by this sender.
 */
	int (*client_cmds[NUM_SENDER_CMDS])(struct sender_command_data*);
};
